@@ -1,4 +1,3 @@
-#!/usr/bin/env python2
 # Terminator by Chris Jones <cmsj@tenshu.net>
 # GPL v2 only
 """terminator.py - class for the master Terminator singleton"""
@@ -10,14 +9,14 @@ gi.require_version('Vte', '2.91')
 from gi.repository import Gtk, Gdk, Vte, GdkX11
 from gi.repository.GLib import GError
 
-import borg
-from borg import Borg
-from config import Config
-from keybindings import Keybindings
-from util import dbg, err, enumerate_descendants
-from factory import Factory
-from cwd import get_pid_cwd
-from version import APP_NAME, APP_VERSION
+from . import borg
+from .borg import Borg
+from .config import Config
+from .keybindings import Keybindings
+from .util import dbg, err, enumerate_descendants
+from .factory import Factory
+from .cwd import get_pid_cwd
+from .version import APP_NAME, APP_VERSION
 
 def eventkey2gdkevent(eventkey):  # FIXME FOR GTK3: is there a simpler way of casting from specific EventKey to generic (union) GdkEvent?
     gdkevent = Gdk.Event.new(eventkey.type)
@@ -267,34 +266,34 @@ class Terminator(Borg):
             count = count + 1
             if count == 1000:
                 err('hit maximum loop boundary. THIS IS VERY LIKELY A BUG')
-            for obj in layout.keys():
+            for obj in list(layout.keys()):
                 if layout[obj]['type'].lower() == 'window':
                     hierarchy[obj] = {}
                     hierarchy[obj]['type'] = 'Window'
                     hierarchy[obj]['children'] = {}
 
                     # Copy any additional keys
-                    for objkey in layout[obj].keys():
-                        if layout[obj][objkey] != '' and not hierarchy[obj].has_key(objkey):
+                    for objkey in list(layout[obj].keys()):
+                        if layout[obj][objkey] != '' and objkey not in hierarchy[obj]:
                             hierarchy[obj][objkey] = layout[obj][objkey]
 
                     objects[obj] = hierarchy[obj]
                     del(layout[obj])
                 else:
                     # Now examine children to see if their parents exist yet
-                    if not layout[obj].has_key('parent'):
+                    if 'parent' not in layout[obj]:
                         err('Invalid object: %s' % obj)
                         del(layout[obj])
                         continue
-                    if objects.has_key(layout[obj]['parent']):
+                    if layout[obj]['parent'] in objects:
                         # Our parent has been created, add ourselves
                         childobj = {}
                         childobj['type'] = layout[obj]['type']
                         childobj['children'] = {}
 
                         # Copy over any additional object keys
-                        for objkey in layout[obj].keys():
-                            if not childobj.has_key(objkey):
+                        for objkey in list(layout[obj].keys()):
+                            if objkey not in childobj:
                                 childobj[objkey] = layout[obj][objkey]
 
                         objects[layout[obj]['parent']]['children'][obj] = childobj
@@ -309,25 +308,25 @@ class Terminator(Borg):
                 raise(ValueError)
             dbg('Creating a window')
             window, terminal = self.new_window()
-            if layout[windef].has_key('position'):
+            if 'position' in layout[windef]:
                 parts = layout[windef]['position'].split(':')
                 if len(parts) == 2:
                     window.move(int(parts[0]), int(parts[1]))
-            if layout[windef].has_key('size'):
+            if 'size' in layout[windef]:
                 parts = layout[windef]['size']
                 winx = int(parts[0])
                 winy = int(parts[1])
                 if winx > 1 and winy > 1:
                     window.resize(winx, winy)
-            if layout[windef].has_key('title'):
+            if 'title' in layout[windef]:
                 window.title.force_title(layout[windef]['title'])
-            if layout[windef].has_key('maximised'):
+            if 'maximised' in layout[windef]:
                 if layout[windef]['maximised'] == 'True':
                     window.ismaximised = True
                 else:
                     window.ismaximised = False
                 window.set_maximised(window.ismaximised)
-            if layout[windef].has_key('fullscreen'):
+            if 'fullscreen' in layout[windef]:
                 if layout[windef]['fullscreen'] == 'True':
                     window.isfullscreen = True
                 else:
@@ -359,7 +358,7 @@ class Terminator(Borg):
                 # For windows with a notebook
                 notebook = window.get_toplevel().get_children()[0]
                 # Cycle through pages by number
-                for page in xrange(0, notebook.get_n_pages()):
+                for page in range(0, notebook.get_n_pages()):
                     # Try and get the entry in the previously saved mapping
                     mapping = window_last_active_term_mapping[window]
                     page_last_active_term = mapping.get(notebook.get_nth_page(page),  None)
@@ -391,10 +390,11 @@ class Terminator(Borg):
 
         # Build list of new windows using prelayout list
         new_win_list = []
-        for window in self.windows:
-            if window not in self.prelayout_windows:
-                new_win_list.append(window)
-        
+        if self.prelayout_windows:
+            for window in self.windows:
+                if window not in self.prelayout_windows:
+                    new_win_list.append(window)
+
         # Make sure all new windows get bumped to the top
         for window in new_win_list:
             window.show()
@@ -474,7 +474,7 @@ class Terminator(Borg):
                 background-color: alpha(%s, %s); }
             """
         profiles = self.config.base.profiles
-        for profile in profiles.keys():
+        for profile in list(profiles.keys()):
             if profiles[profile]['use_theme_colors']:
                 # Create a dummy window/vte and realise it so it has correct
                 # values to read from
@@ -501,7 +501,7 @@ class Terminator(Borg):
             css += template % (munged_profile, bgcolor, bgalpha)
 
         style_provider = Gtk.CssProvider()
-        style_provider.load_from_data(css)
+        style_provider.load_from_data(css.encode('utf-8'))
         self.style_providers.append(style_provider)
 
         # Attempt to load some theme specific stylistic tweaks for appearances
@@ -537,19 +537,19 @@ class Terminator(Borg):
 
         # Size the GtkPaned splitter handle size.
         css = ""
-        if self.config['handle_size'] in xrange(0, 21):
+        if self.config['handle_size'] in range(0, 21):
             css += """
                 .terminator-terminal-window GtkPaned,
                 .terminator-terminal-window paned {
                     -GtkPaned-handle-size: %s; }
                 """ % self.config['handle_size']
         style_provider = Gtk.CssProvider()
-        style_provider.load_from_data(css)
+        style_provider.load_from_data(css.encode('utf-8'))
         self.style_providers.append(style_provider)
 
         # Apply the providers, incrementing priority so they don't cancel out
         # each other
-        for idx in xrange(0, len(self.style_providers)):
+        for idx in range(0, len(self.style_providers)):
             Gtk.StyleContext.add_provider_for_screen(
                 Gdk.Screen.get_default(),
                 self.style_providers[idx],
