@@ -1,30 +1,33 @@
-#!/usr/bin/python
 # Terminator by Chris Jones <cmsj@tenshu.net>
 # GPL v2 only
 """window.py - class for the main Terminator window"""
 
 import copy
-import pygtk
-pygtk.require('2.0')
-import gobject
-import gtk
+import time
+import uuid
+import gi
+from gi.repository import GObject
+from gi.repository import Gtk, Gdk, GdkX11
 
-from util import dbg, err
-import util
-from translation import _
-from version import APP_NAME
-from container import Container
-from factory import Factory
-from terminator import Terminator
+from .util import dbg, err, make_uuid, display_manager
+from . import util
+from .translation import _
+from .version import APP_NAME
+from .container import Container
+from .factory import Factory
+from .terminator import Terminator
 
-try:
-    import keybinder
-except ImportError:
-    err('Warning: python-keybinder is not installed. This means the \
+if display_manager() == 'X11':
+    try:
+        gi.require_version('Keybinder', '3.0')
+        from gi.repository import Keybinder
+        Keybinder.init()
+    except (ImportError, ValueError):
+        err('Unable to load Keybinder module. This means the \
 hide_window shortcut will be unavailable')
 
 # pylint: disable-msg=R0904
-class Window(Container, gtk.Window):
+class Window(Container, Gtk.Window):
     """Class implementing a top-level Terminator window"""
 
     terminator = None
@@ -33,16 +36,22 @@ class Window(Container, gtk.Window):
     ismaximised = None
     hidebound = None
     hidefunc = None
+    losefocus_time = 0
+    position = None
+    ignore_startup_show = None
+    set_pos_by_ratio = None
+    last_active_term = None
+    preventHide = None
 
     zoom_data = None
 
     term_zoomed = False
     __gproperties__ = {
-            'term_zoomed': (gobject.TYPE_BOOLEAN,
+            'term_zoomed': (GObject.TYPE_BOOLEAN,
                             'terminal zoomed',
                             'whether the terminal is zoomed',
                             False,
-                            gobject.PARAM_READWRITE)
+                            GObject.PARAM_READWRITE)
     }
 
     def __init__(self):
@@ -51,32 +60,42 @@ class Window(Container, gtk.Window):
         self.terminator.register_window(self)
 
         Container.__init__(self)
-        gtk.Window.__init__(self)
-        gobject.type_register(Window)
+        GObject.GObject.__init__(self)
+        GObject.type_register(Window)
         self.register_signals(Window)
 
-        self.set_property('allow-shrink', True)
-        self.apply_icon()
+        self.get_style_context().add_class("terminator-terminal-window")
+
+#        self.set_property('allow-shrink', True)  # FIXME FOR GTK3, or do we need this actually?
+        icon_to_apply=''
 
         self.register_callbacks()
         self.apply_config()
 
         self.title = WindowTitle(self)
         self.title.update()
+        
+        self.preventHide = False
 
         options = self.config.options_get()
         if options:
-            if options.forcedtitle is not None:
+            if options.forcedtitle:
                 self.title.force_title(options.forcedtitle)
 
-            if options.role is not None:
+            if options.role:
                 self.set_role(options.role)
+            
+            if options.forcedicon is not None:
+                icon_to_apply = options.forcedicon
 
-            if options.geometry is not None:
+            if options.geometry:
                 if not self.parse_geometry(options.geometry):
                     err('Window::__init__: Unable to parse geometry: %s' % 
                             options.geometry)
 
+        self.apply_icon(icon_to_apply)
+        self.pending_set_rough_geometry_hint = False
+
     def do_get_property(self, prop):
         """Handle gobject getting a property"""
         if prop.name in ['term_zoomed', 'term-zoomed']:
@@ -103,18 +122,20 @@ class Window(Container, gtk.Window):
 
         # Attempt to grab a global hotkey for hiding the window.
         # If we fail, we'll never hide the window, iconifying instead.
-        try:
-            self.hidebound = keybinder.bind(
-                self.config['keybindings']['hide_window'],
-                self.on_hide_window)
-        except (KeyError, NameError):
-            pass
+        if self.config['keybindings']['hide_window'] != None:
+            if display_manager() == 'X11':
+                try:
+                    self.hidebound = Keybinder.bind(
+                        self.config['keybindings']['hide_window'].replace('<Shift>',''),
+                        self.on_hide_window)
+                except (KeyError, NameError):
+                    pass
 
-        if not self.hidebound:
-            err('Unable to bind hide_window key, another instance/window has it.')
-            self.hidefunc = self.iconify
-        else:
-            self.hidefunc = self.hide
+                if not self.hidebound:
+                    err('Unable to bind hide_window key, another instance/window has it.')
+                    self.hidefunc = self.iconify
+                else:
+                    self.hidefunc = self.hide
 
     def apply_config(self):
         """Apply various configuration options"""
@@ -123,6 +144,9 @@ class Window(Container, gtk.Window):
         fullscreen = self.config['window_state'] == 'fullscreen'
         hidden = self.config['window_state'] == 'hidden'
         borderless = self.config['borderless']
+        skiptaskbar = self.config['hide_from_taskbar']
+        alwaysontop = self.config['always_on_top']
+        sticky = self.config['sticky']
 
         if options:
             if options.maximise:
@@ -137,22 +161,38 @@ class Window(Container, gtk.Window):
         self.set_fullscreen(fullscreen)
         self.set_maximised(maximise)
         self.set_borderless(borderless)
+        self.set_always_on_top(alwaysontop)
         self.set_real_transparency()
+        self.set_sticky(sticky)
         if self.hidebound:
             self.set_hidden(hidden)
+            self.set_skip_taskbar_hint(skiptaskbar)
         else:
             self.set_iconified(hidden)
 
-    def apply_icon(self):
+    def apply_icon(self, requested_icon):
         """Set the window icon"""
-        icon_theme = gtk.IconTheme()
+        icon_theme = Gtk.IconTheme.get_default()
+        icon_name_list = [APP_NAME]   # disable self.wmclass_name, n/a in GTK3
 
-        try:
-            icon = icon_theme.load_icon(APP_NAME, 48, 0)
-        except (NameError, gobject.GError):
-            dbg('Unable to load 48px Terminator icon')
-            icon = self.render_icon(gtk.STOCK_DIALOG_INFO, gtk.ICON_SIZE_BUTTON)
+        if requested_icon:
+            try:
+                self.set_icon_from_file(requested_icon)
+                return
+            except (NameError, GObject.GError):
+                dbg('Unable to load %s icon as file' % (repr(requested_icon)))
 
+            icon_name_list.insert(0, requested_icon)
+
+        for icon_name in icon_name_list:
+            # Test if the icon is available first
+            if icon_theme.lookup_icon(icon_name, 48, 0):
+                self.set_icon_name(icon_name)
+                return # Success! We're done.
+            else:
+                dbg('Unable to load %s icon' % (icon_name))
+
+        icon = self.render_icon(Gtk.STOCK_DIALOG_INFO, Gtk.IconSize.BUTTON)
         self.set_icon(icon)
 
     def on_key_press(self, window, event):
@@ -169,11 +209,9 @@ class Window(Container, gtk.Window):
                 self.set_fullscreen(not self.isfullscreen)
             elif mapping == 'close_window':
                 if not self.on_delete_event(window,
-                        gtk.gdk.Event(gtk.gdk.DELETE)):
+                        Gdk.Event.new(Gdk.EventType.DELETE)):
                     self.on_destroy_event(window,
-                            gtk.gdk.Event(gtk.gdk.DESTROY))
-            elif mapping == 'new_tab':
-                self.tab_new(self.get_focussed_terminal())
+                            Gdk.Event.new(Gdk.EventType.DESTROY))
             else:
                 return(False)
             return(True)
@@ -189,9 +227,20 @@ class Window(Container, gtk.Window):
         for terminal in self.get_visible_terminals():
             terminal.on_window_focus_out()
 
+        self.losefocus_time = time.time()
+
+        if self.preventHide:
+            self.preventHide = False
+        else:
+            if self.config['hide_on_lose_focus'] and self.get_property('visible'):
+                self.position = self.get_position()
+                self.hidefunc()
+
     def on_focus_in(self, window, event):
         """Focus has entered the window"""
         self.set_urgency_hint(False)
+        if not self.terminator.doing_layout:
+            self.terminator.last_active_window = self.uuid
         # FIXME: Cause the terminal titlebars to update here
 
     def is_child_notebook(self):
@@ -202,6 +251,7 @@ class Window(Container, gtk.Window):
     def tab_new(self, widget=None, debugtab=False, _param1=None, _param2=None):
         """Make a new tab"""
         cwd = None
+        profile = None
 
         if self.get_property('term_zoomed') == True:
             err("You can't create a tab while a terminal is maximised/zoomed")
@@ -209,17 +259,25 @@ class Window(Container, gtk.Window):
 
         if widget:
             cwd = widget.get_cwd()
+            profile = widget.get_profile()
+
         maker = Factory()
         if not self.is_child_notebook():
+            dbg('Making a new Notebook')
             notebook = maker.make('Notebook', window=self)
-        self.get_child().newtab(debugtab, cwd=cwd)
+        self.show()
+        self.present()
+        return self.get_child().newtab(debugtab, cwd=cwd, profile=profile)
 
     def on_delete_event(self, window, event, data=None):
         """Handle a window close request"""
         maker = Factory()
         if maker.isinstance(self.get_child(), 'Terminal'):
-            dbg('Window::on_delete_event: Only one child, closing is fine')
-            return(False)
+            if self.get_property('term_zoomed') == True:
+                return(self.confirm_close(window, _('window')))
+            else:
+                dbg('Window::on_delete_event: Only one child, closing is fine')
+                return(False)
         elif maker.isinstance(self.get_child(), 'Container'):
             return(self.confirm_close(window, _('window')))
         else:
@@ -228,10 +286,8 @@ class Window(Container, gtk.Window):
     def confirm_close(self, window, type):
         """Display a confirmation dialog when the user is closing multiple
         terminals in one window"""
-        dialog = self.construct_confirm_close(window, type)
-        result = dialog.run()
-        dialog.destroy()
-        return(not (result == gtk.RESPONSE_ACCEPT))
+        
+        return(not (self.construct_confirm_close(window, type) == Gtk.ResponseType.ACCEPT))
 
     def on_destroy_event(self, widget, data=None):
         """Handle window destruction"""
@@ -245,18 +301,35 @@ class Window(Container, gtk.Window):
 
     def on_hide_window(self, data=None):
         """Handle a request to hide/show the window"""
-        # FIXME: Implement or drop, or explain why its empty
-        pass
+
+        if not self.get_property('visible'):
+            #Don't show if window has just been hidden because of
+            #lost focus
+            if (time.time() - self.losefocus_time < 0.1) and \
+                self.config['hide_on_lose_focus']:
+                return
+            if self.position:
+                self.move(self.position[0], self.position[1])
+            self.show()
+            self.grab_focus()
+            try:
+                t = GdkX11.x11_get_server_time(self.get_window())
+            except (TypeError, AttributeError):
+                t = 0
+            self.get_window().focus(t)
+        else:
+            self.position = self.get_position()
+            self.hidefunc()
 
     # pylint: disable-msg=W0613
     def on_window_state_changed(self, window, event):
         """Handle the state of the window changing"""
         self.isfullscreen = bool(event.new_window_state & 
-                                 gtk.gdk.WINDOW_STATE_FULLSCREEN)
+                                 Gdk.WindowState.FULLSCREEN)
         self.ismaximised = bool(event.new_window_state &
-                                 gtk.gdk.WINDOW_STATE_MAXIMIZED)
-        dbg('Window::on_window_state_changed: fullscreen=%s, maximised=%s' %
-            (self.isfullscreen, self.ismaximised))
+                                 Gdk.WindowState.MAXIMIZED)
+        dbg('Window::on_window_state_changed: fullscreen=%s, maximised=%s' \
+                % (self.isfullscreen, self.ismaximised))
 
         return(False)
 
@@ -280,13 +353,24 @@ class Window(Container, gtk.Window):
 
     def set_hidden(self, value):
         """Set the visibility of the window from the supplied value"""
-        # FIXME: Implement or drop this
-        pass
+        if value == True:
+            self.ignore_startup_show = True
+        else:
+            self.ignore_startup_show = False
 
     def set_iconified(self, value):
-        """Set the minimised state of the window from the value"""
-        # FIXME: Implement or drop this
-        pass
+        """Set the minimised state of the window from the supplied value"""
+        if value == True:
+            self.iconify()
+
+    def set_always_on_top(self, value):
+        """Set the always on top window hint from the supplied value"""
+        self.set_keep_above(value)
+
+    def set_sticky(self, value):
+        """Set the sticky hint from the supplied value"""
+        if value == True:
+            self.stick()
 
     def set_real_transparency(self, value=True):
         """Enable RGBA if supported on the current screen"""
@@ -295,19 +379,29 @@ class Window(Container, gtk.Window):
 
         screen = self.get_screen()
         if value:
-            dbg('setting rgba colormap')
-            colormap = screen.get_rgba_colormap()
-        else:
-            dbg('setting rgb colormap')
-            colormap = screen.get_rgb_colormap()
+            dbg('setting rgba visual')
+            visual = screen.get_rgba_visual()
+            if visual:
+                self.set_visual(visual)
+    
+    def show(self, startup=False):
+        """Undo the startup show request if started in hidden mode"""
+        #Present is necessary to grab focus when window is hidden from taskbar.
+        #It is important to call present() before show(), otherwise the window
+        #won't be brought to front if an another application has the focus.
+        #Last note: present() will implicitly call Gtk.Window.show()
+        self.present()
 
-        if colormap:
-            self.set_colormap(colormap)
+        #Window must be shown, then hidden for the hotkeys to be registered
+        if (self.ignore_startup_show and startup == True):
+            self.position = self.get_position()
+            self.hide()
 
-    def add(self, widget):
-        """Add a widget to the window by way of gtk.Window.add()"""
+
+    def add(self, widget, metadata=None):
+        """Add a widget to the window by way of Gtk.Window.add()"""
         maker = Factory()
-        gtk.Window.add(self, widget)
+        Gtk.Window.add(self, widget)
         if maker.isinstance(widget, 'Terminal'):
             signals = {'close-term': self.closeterm,
                        'title-change': self.title.set_title,
@@ -316,8 +410,10 @@ class Window(Container, gtk.Window):
                        'unzoom': self.unzoom,
                        'tab-change': self.tab_change,
                        'group-all': self.group_all,
+                       'group-all-toggle': self.group_all_toggle,
                        'ungroup-all': self.ungroup_all,
                        'group-tab': self.group_tab,
+                       'group-tab-toggle': self.group_tab_toggle,
                        'ungroup-tab': self.ungroup_tab,
                        'move-tab': self.move_tab,
                        'tab-new': [self.tab_new, widget],
@@ -334,8 +430,8 @@ class Window(Container, gtk.Window):
             widget.grab_focus()
 
     def remove(self, widget):
-        """Remove our child widget by way of gtk.Window.remove()"""
-        gtk.Window.remove(self, widget)
+        """Remove our child widget by way of Gtk.Window.remove()"""
+        Gtk.Window.remove(self, widget)
         self.disconnect_child(widget)
         return(True)
 
@@ -369,11 +465,20 @@ class Window(Container, gtk.Window):
             container = maker.make('VPaned')
         else:
             container = maker.make('HPaned')
+        
+        self.set_pos_by_ratio = True
 
         if not sibling:
             sibling = maker.make('Terminal')
             sibling.set_cwd(cwd)
+            if self.config['always_split_with_profile']:
+                sibling.force_set_profile(None, widget.get_profile())
             sibling.spawn_child()
+            if widget.group and self.config['split_to_group']:
+                sibling.set_group(None, widget.group)
+        elif self.config['always_split_with_profile']:
+            sibling.force_set_profile(None, widget.get_profile())
+
         self.add(container)
         container.show_all()
 
@@ -384,6 +489,12 @@ class Window(Container, gtk.Window):
         for term in order:
             container.add(term)
         container.show_all()
+        
+        while Gtk.events_pending():
+            Gtk.main_iteration_do(False)
+        sibling.grab_focus()
+        self.set_pos_by_ratio = False
+
 
     def zoom(self, widget, font_scale=True):
         """Zoom a terminal widget"""
@@ -428,11 +539,39 @@ class Window(Container, gtk.Window):
         self.zoom_data = None
         self.set_property('term_zoomed', False)
 
+    def rotate(self, widget, clockwise):
+        """Rotate children in this window"""
+        self.set_pos_by_ratio = True
+        maker = Factory()
+        child = self.get_child()
+
+        # If our child is a Notebook, reset to work from its visible child
+        if maker.isinstance(child, 'Notebook'):
+            pagenum = child.get_current_page()
+            child = child.get_nth_page(pagenum)
+
+        if maker.isinstance(child, 'Paned'):
+            parent = child.get_parent()
+            # Need to get the allocation before we remove the child,
+            # otherwise _sometimes_ we get incorrect values.
+            alloc = child.get_allocation()
+            parent.remove(child)
+            child.rotate_recursive(parent, alloc.width, alloc.height, clockwise)
+
+            self.show_all()
+            while Gtk.events_pending():
+                Gtk.main_iteration_do(False)
+            widget.grab_focus()
+
+        self.set_pos_by_ratio = False
+
     def get_visible_terminals(self):
         """Walk down the widget tree to find all of the visible terminals.
         Mostly using Container::get_visible_terminals()"""
         terminals = {}
-        maker = Factory()
+        if not hasattr(self, 'cached_maker'):
+            self.cached_maker = Factory()
+        maker = self.cached_maker
         child = self.get_child()
 
         if not child:
@@ -460,10 +599,26 @@ class Window(Container, gtk.Window):
                 return(terminal)
         return(None)
 
+    def deferred_set_rough_geometry_hints(self):
+        # no parameters are used in set_rough_geometry_hints, so we can
+        # use the set_rough_geometry_hints
+        if self.pending_set_rough_geometry_hint == True:
+            return
+        self.pending_set_rough_geometry_hint = True
+        GObject.idle_add(self.do_deferred_set_rough_geometry_hints)
+
+    def do_deferred_set_rough_geometry_hints(self):
+        self.pending_set_rough_geometry_hint = False
+        self.set_rough_geometry_hints()
+
     def set_rough_geometry_hints(self):
         """Walk all the terminals along the top and left edges to fake up how
         many columns/rows we sort of have"""
-        maker = Factory()
+        if self.ismaximised == True:
+            return
+        if not hasattr(self, 'cached_maker'):
+            self.cached_maker = Factory()
+        maker = self.cached_maker
         if maker.isinstance(self.get_child(), 'Notebook'):
             dbg("We don't currently support geometry hinting with tabs")
             return
@@ -501,8 +656,12 @@ class Window(Container, gtk.Window):
         dbg('setting geometry hints: (ewidth:%s)(eheight:%s),\
 (fwidth:%s)(fheight:%s)' % (extra_width, extra_height, 
                             font_width, font_height))
-        self.set_geometry_hints(self, -1, -1, -1, -1, extra_width,
-                extra_height, font_width, font_height, -1.0, -1.0)
+        geometry = Gdk.Geometry()
+        geometry.base_width = extra_width
+        geometry.base_height = extra_height
+        geometry.width_inc = font_width
+        geometry.height_inc = font_height
+        self.set_geometry_hints(self, geometry, Gdk.WindowHints.BASE_SIZE | Gdk.WindowHints.RESIZE_INC)
 
     def tab_change(self, widget, num=None):
         """Change to a specific tab"""
@@ -538,19 +697,29 @@ class Window(Container, gtk.Window):
         # change
         child.set_current_page(child.get_current_page())
 
-    # FIXME: All of these (un)group_(all|tab) methods need refactoring work
+    def set_groups(self, new_group,  term_list):
+        """Set terminals in term_list to new_group"""
+        for terminal in term_list:
+            terminal.set_group(None, new_group)
+        self.terminator.focus_changed(self.terminator.last_focused_term)
+
     def group_all(self, widget):
         """Group all terminals"""
         # FIXME: Why isn't this being done by Terminator() ?
         group = _('All')
         self.terminator.create_group(group)
-        for terminal in self.terminator.terminals:
-            terminal.set_group(None, group)
+        self.set_groups(group, self.terminator.terminals)
+
+    def group_all_toggle(self, widget):
+        """Toggle grouping to all"""
+        if widget.group == 'All':
+            self.ungroup_all(widget)
+        else:
+            self.group_all(widget)
 
     def ungroup_all(self, widget):
         """Ungroup all terminals"""
-        for terminal in self.terminator.terminals:
-            terminal.set_group(None, None)
+        self.set_groups(None, self.terminator.terminals)
 
     def group_tab(self, widget):
         """Group all terminals in the current tab"""
@@ -567,8 +736,14 @@ class Window(Container, gtk.Window):
             if group not in self.terminator.groups:
                 break
             pagenum += 1
-        for terminal in self.get_visible_terminals():
-            terminal.set_group(None, group)
+        self.set_groups(group, self.get_visible_terminals())
+
+    def group_tab_toggle(self, widget):
+        """Blah"""
+        if widget.group and widget.group[:4] == 'Tab ':
+            self.ungroup_tab(widget)
+        else:
+            self.group_tab(widget)
 
     def ungroup_tab(self, widget):
         """Ungroup all terminals in the current tab"""
@@ -579,8 +754,7 @@ class Window(Container, gtk.Window):
             dbg('note in a notebook, refusing to ungroup tab')
             return
         
-        for terminal in self.get_visible_terminals():
-            terminal.set_group(None, None)
+        self.set_groups(None, self.get_visible_terminals())
 
     def move_tab(self, widget, direction):
         """Handle a keyboard shortcut for moving tab positions"""
@@ -643,12 +817,13 @@ class Window(Container, gtk.Window):
             possibles = []
 
             # Get the co-ordinate of the appropriate edge for this direction
-            edge = util.get_edge(allocation, direction)
+            edge, p1, p2 = util.get_edge(allocation, direction)
             # Find all visible terminals which are, in their entirity, in the
-            # direction we want to move
+            # direction we want to move, and are at least partially spanning
+            # p1 to p2
             for term in layout:
                 rect = layout[term]
-                if util.get_nav_possible(edge, rect, direction):
+                if util.get_nav_possible(edge, rect, direction, p1, p2):
                     possibles.append(term)
 
             if len(possibles) == 0:
@@ -661,14 +836,14 @@ class Window(Container, gtk.Window):
             for term in possibles:
                 rect = layout[term]
                 offsets[term] = util.get_nav_offset(edge, rect, direction)
-            keys = offsets.values()
+            keys = list(offsets.values())
             keys.sort()
-            winners = [k for k, v in offsets.iteritems() if v == keys[0]]
+            winners = [k for k, v in offsets.items() if v == keys[0]]
             next = terminals.index(winners[0])
 
             if len(winners) > 1:
                 # Break an n-way tie using the cursor position
-                term_alloc = terminal.allocation
+                term_alloc = terminal.get_allocation()
                 cursor_x = term_alloc.x + term_alloc.width / 2
                 cursor_y = term_alloc.y + term_alloc.height / 2
 
@@ -686,7 +861,7 @@ class Window(Container, gtk.Window):
 
     def create_layout(self, layout):
         """Apply any config items from our layout"""
-        if not layout.has_key('children'):
+        if 'children' not in layout:
             err('layout describes no children: %s' % layout)
             return
         children = layout['children']
@@ -695,7 +870,7 @@ class Window(Container, gtk.Window):
             err('incorrect number of children for Window: %s' % layout)
             return
 
-        child = children[children.keys()[0]]
+        child = children[list(children.keys())[0]]
         terminal = self.get_children()[0]
         dbg('Making a child of type: %s' % child['type'])
         if child['type'] == 'VPaned':
@@ -704,6 +879,10 @@ class Window(Container, gtk.Window):
             self.split_axis(terminal, False)
         elif child['type'] == 'Notebook':
             self.tab_new()
+            i = 2
+            while i < len(child['children']):
+                self.tab_new()
+                i = i + 1
         elif child['type'] == 'Terminal':
             pass
         else:
@@ -712,6 +891,12 @@ class Window(Container, gtk.Window):
 
         self.get_children()[0].create_layout(child)
 
+        if 'last_active_term' in layout and layout['last_active_term'] not in ['', None]:
+            self.last_active_term = make_uuid(layout['last_active_term'])
+
+        if 'last_active_window' in layout and layout['last_active_window'] == 'True':
+            self.terminator.last_active_window = self.uuid
+
 class WindowTitle(object):
     """Class to handle the setting of the window title"""
 
